/* ************ COPYRIGHT 2011, H. G. Katzgraber. ********************* */
/*                                                                      */
/* EX 6:        One-dimensional Ising Model with Monte Carlo            */
/*		periodic boundary conditions: system on closed loop	*/
/*		change parameters in header file ising.h		*/
/*                                                                      */
/*              NOTE: this program ignores autocorrelation effects!!!   */
/* INSTR:       Compile with Makefile via 'make', execute with ./runme  */

#include "ising.h"

void generate_boundary_cond(int **nb_matrix);
void update(int *spins, int **nb_matrix, double *energy, double temp);

int main(void)
{
    int 	*spins, **nb_matrix, i, j, k, m, l;
    double 	tmp, e, u, ave_u, ave_sq_u, err_u;
    long 	seed;
    
    seed = seedgen();
    setr1279(seed);
    
    spins  	= ivector(1,N);		/* numerical recipces vector 	*/

	/* lookup matrix for neighbors -------------------------------- */

    nb_matrix   = imatrix(1,2,1,N);	/* numerical recipes matrix 	*/

    for (i = 1; i <= N; i++) { 
	if (i == 1) {                   /* if first spin, left nb is N  */
	    nb_matrix[1][i] = N;
	    nb_matrix[2][i] = i + 1;
	}
	else if (i == N){               /* if last spin, right nb is 1  */
	    nb_matrix[1][i] = i - 1;
	    nb_matrix[2][i] = 1;
	}
	else {
	    nb_matrix[1][i] = i - 1;    /* any other spin in the middle */
	    nb_matrix[2][i] = i + 1;
	}
    }

    printf("# 1D Ising model, N = %d, %d MCS, %d SAMPL\n",N,SWEEPS,SAMPLES);
    printf("# Temp           Energy/spin    Error\n");

    	/* main routine -----------------------------------------------	*/

    for(j = 1; j <= M; j++){		/* loop over temperatures	*/
	tmp = ((double) DT*j);		/* compute temperature value	*/

	ave_u    = 0.0;
	ave_sq_u = 0.0;
	
	for(m = 1; m <= SAMPLES; m++){	/* average over runs for stats	*/

	    for(i = 1; i <= N; i++){	/* initialize spins to all up	*/
		 spins[i] = 1;
	    }
	    e = -1.0*((double) N);	/* initialize energy accordingly*/

	    u = 0.0;
	    
	    for(k = 1; k <= TRANS; k++){ /* equilibration sweeps	*/
		update(spins,nb_matrix,&e,tmp);
	    }
	    for(i = 1; i <= SWEEPS; i++){/* measurement sweeps		*/
		update(spins,nb_matrix,&e,tmp);
		u += e;
	    }
	    u /= ((double) (SWEEPS*N));
	    ave_u    += u;
	    ave_sq_u += (u*u);
	}
	
	ave_u    /= ((double) SAMPLES);
	ave_sq_u /= ((double) SAMPLES);
	err_u     = calc_err(ave_sq_u, ave_u, SAMPLES);

	printf("%e\t%e\t%e\n",tmp,ave_u,err_u);
    }
    
    free_ivector(spins,1,N);
    free_imatrix(nb_matrix,1,2,1,N);
    
    return EXIT_SUCCESS;
}

