/* ************ COPYRIGHT 2011, H. G. Katzgraber. ********************* */
/*                                                                      */
/*              Routine attempts N Monte Carlo spin updates             */
/*		NOTE: in 1D Metropolis MUST have a random spin slected. */
/*		      Metropolis and sequential updating are not ergodic*/
/*		      The alternative is to use heat bath (see below)   */

#include "ising.h"
#include "r1279.h"

void update(int *spins, int **nb_matrix, double *energy, double temp){

    int 	i, delta, s;

    for (i = 1; i <= N; i++){

	/* Metropolis version of the code, random update is a MUST	*/


	s = ir1279range(1,N); /* selects random number in [1,N]		*/

	delta = 2.0*spins[s]*(spins[nb_matrix[1][s]] + spins[nb_matrix[2][s]]);

	if (r1279() < (exp(-1.0*((double) delta)/temp))){
	    spins[s]  = -spins[s];			
	    *energy  += ((double) delta);
	}

	/* heat bath version of the code, sequential update OK		*/

	/*
	delta = 2.0*spins[i]*(spins[nb_matrix[1][i]] + spins[nb_matrix[2][i]]);

	if (r1279() < 1.0/(exp(((double) delta)/temp) + 1.0)){
	    spins[i]  = -spins[i];
	    *energy  += ((double) delta);
	}
	*/
    }
}

